/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of Simonstrator.KOM.
 * 
 * Simonstrator.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tudarmstadt.maki.simonstrator.api.component.mape.stateexchange;

import de.tudarmstadt.maki.simonstrator.api.component.HostComponent;

/**
 * Specifies the interface of a component that is used by the
 * {@link StateExchangeComponent} to retrieve information of other components on
 * this node. Implemented by other components like e.g. pub/sub, streaming etc.
 * from which metrics can be retrieved.
 * 
 * @author Nils Richerzhagen
 * 
 */
public interface InfoProviderComponent extends HostComponent {

	/**
	 * Returns true, if this info provider can satisfy the given request.
	 * 
	 * @param metricClass
	 * @param request
	 * @return
	 */
	public <T extends RoutingInfo> boolean canProvideInfo(Class<T> metricClass,
			MetricRequest request);

	/**
	 * Give back requested metric object of the requested metric class.
	 * Pull-based, therefore no listener has to be installed.
	 * 
	 * @param metricClass
	 * @return
	 */
	public <T extends RoutingInfo> T getInfo(Class<T> metricClass,
			MetricRequest metricRequest)
			throws MetricNotAvailableException;

	/**
	 * Request push notifications on the given listener based on the data of the
	 * given MonitorignRequest (the included respective metric).
	 * 
	 * @param listener
	 * @param request
	 */
	// Maybe build a requestData object with which the original Monitoring
	// Request can be overhanded
	public <T extends RoutingInfo> void requestInfoUpdates(
			InfoProviderUpdateListener listener, MetricRequest metricRequest)
			throws MetricNotAvailableException;

	/**
	 * Remove the given listener - stop updating.
	 * 
	 * @param listener
	 */
	public void removeInfoUpdates(InfoProviderUpdateListener listener);

	/**
	 * Description-Object of this InfoProvider
	 * 
	 * @return
	 */
	public InfoProviderDescription getInfoProviderDescription();

}
