/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of Simonstrator.KOM.
 * 
 * Simonstrator.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tudarmstadt.maki.simonstrator.api.component.mape.stateexchange;


/**
 * MonitoringRequest, retrieve an object for manipulation through the
 * {@link StateExchangeComponent}. Used from components to request monitoring
 * data from {@link StateExchangeComponent}, it includes the way of retrieving
 * this data (single-pull, periodic pull and push); and the MetricRequest
 * 
 * 1. single-pull: only return the {@link MonitoringRequest}
 * 
 * 2. periodic-pull: setInterval & setPeriodic must be set
 * 
 * 3. push: only setPush must be set
 * 
 * @author Nils Richerzhagen
 * 
 */
public interface MonitoringRequest<T extends RoutingInfo> extends MetricRequest {

	/**
	 * Time between updates of the listener - not accurate (at least not on
	 * android), so do not rely on the listener for periodic operations.
	 * 
	 * @param interval
	 *            in Timer.units
	 */
	public void setInterval(long interval);

	/**
	 * Set <b> true </b> for one single pull update.
	 * 
	 * @param single
	 */
	public void setSinglePull(boolean single);

	/**
	 * Set <b> true </b> for periodic updates.
	 * 
	 * @param periodic
	 */
	public void setPeriodic(boolean periodic);

	/**
	 * Set <b> true </b> for push based updates.
	 * 
	 * @param push
	 */
	public void setPush(boolean push);

	/**
	 * Get the MetricClass of that MonitoringRequest.
	 * 
	 * @return
	 */
	public Class<T> getMetricClass();

	/**
	 * Set MetricClass
	 */
	public void setMetricClass(Class<T> metricClass);
}
