/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of Simonstrator.KOM.
 * 
 * Simonstrator.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tudarmstadt.maki.simonstrator.api.component.mape.stateexchange;

import java.util.List;

import de.tudarmstadt.maki.simonstrator.api.component.HostComponent;

/**
 * Specifies the interface of a component that is used for inter-component
 * communication. Inter-component communication is build like a bus system on
 * which components can ask for (monitoring) metrics that they need. The other
 * way around this interface request the metrics from all available components
 * that implement the InfoProvider interface.
 * 
 * Thus, this interface is used for scanning for components implementing the
 * required metric. <b>NO</b> handshake or advertising of the components
 * themselves is used.
 * 
 * @author Nils Richerzhagen
 * 
 */
public interface StateExchangeComponent extends HostComponent {

	/**
	 * Handshake, part one: retrieve a list of InfoProviders that can provide
	 * the respective metric.
	 * 
	 * @param req
	 * @return
	 */
	public List<InfoProviderDescription> getInfoProvidersFor(
			MonitoringRequest req);

	/**
	 * Start receiving metric updates for the specified metricClass with the
	 * provided request parameters on the provided listener.
	 * 
	 * Used for pulling, periodic pulling and push-based notifications.
	 * 
	 * @param request
	 * @param provider
	 *            one of the providers returned by getInfoProvidersFor(..)
	 * @param metricClass
	 * @param listener
	 */
	public <T extends RoutingInfo> void getState(MonitoringRequest<T> request,
			InfoProviderDescription provider,
			StateExchangeUpdateListener listener)
			throws MetricNotAvailableException;

	/**
	 * Used to release the push update listener on this component for the
	 * calling component.
	 * 
	 * @param listener
	 *            the listener that should be released
	 * @param metricClass
	 */
	public void removeStateUpdates(StateExchangeUpdateListener listener);

	/**
	 * Retrieve a {@link MonitoringRequest} object for manipulation and usage
	 * with requestInfoUpdate for the specified metricClass.
	 * 
	 * @param metricClass
	 * @return
	 */
	public <T extends RoutingInfo> MonitoringRequest<T> getMonitoringRequest(
			Class<T> metricClass);

}
