package de.tudarmstadt.maki.simonstrator.api.component.resourcedistribution;

import de.tudarmstadt.maki.simonstrator.api.common.UniqueID;
import de.tudarmstadt.maki.simonstrator.api.component.HostComponent;

/**
 * Host components implementing this interface will be notified when resources have been collected by a consumer
 * at a {@link ResourceDistributor}. This can e.g. be used by a consumer to update resource information in the store.
 *
 * @author Tim Feuerbach
 */
public interface ResourceCollectionListener extends HostComponent {
    public static final double BEACON_NOT_AVAILABLE = -1;

    /**
     * Called when resources have been collected by the host implementing this interface.
     *
     * @param beaconID           the {@link ResourceDistributor#getBeaconID() unique ResourceDistributor ID}
     * @param beaconSequenceID   the current beacon sequence ID after resources were transferred to the consumer.
     * @param resourcesObtained  the number of resources obtained from the distributor
     * @param resourcesRemaining the number of resources remaining at the distributor
     *                           <b>NOTE:</b> will be {@value #BEACON_NOT_AVAILABLE} when the resource distributor
     *                           could not be found at the specified location
     */
    public void resourcesCollected(UniqueID beaconID, int beaconSequenceID, double resourcesObtained, double
            resourcesRemaining);
}
