package de.tudarmstadt.maki.simonstrator.api.component.resourcedistribution;

import de.tudarmstadt.maki.simonstrator.api.common.UniqueID;
import de.tudarmstadt.maki.simonstrator.api.component.HostComponent;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.location.Location;

/**
 * Interface for components that consume resources depending on two kind of states.
 *
 * TODO implement movement model
 */
public interface ResourceConsumer extends HostComponent {
    /**
     * Returns the current node state
     */
    ResourceConsumptionState getState();

    /**
     * Set the current node state.
     * <p/>
     * <b>Any implementation must ensure that state OFFLINE cannot be left.</b>
     */
    void setState(ResourceConsumptionState state);

    /**
     * Will be notified about chanes of the consumer's {@link ResourceConsumptionState}
     *
     * @see #removeStateChangeListener(ResourceStateChangeListener)
     */
    void addStateChangeListener(ResourceStateChangeListener listener);

    /**
     * Removes the resource state listener, if present
     *
     * @see #addStateChangeListener(ResourceStateChangeListener)
     */
    void removeStateChangeListener(ResourceStateChangeListener listener);

    /**
     * If the node is in state {@link ResourceConsumptionState#HEADING_FOR_RESOURCE}, this method returns the location
     * of a known distributor the node is heading for. Otherwise, returns null.
     * @return
     */
    Location getTarget();

    /**
     * Return the target {@link ResourceDistributor}'s ID or null if there node is currently not heading for a
     * resource.
     */
    UniqueID getTargetBeaconID();

    /**
     * Returns how many resources a node should take upon arrival at a distributor. This value must be zero in states
     * different from {@link ResourceConsumptionState#HEADING_FOR_RESOURCE}.
     */
    double getDemand();
}
