package de.tudarmstadt.maki.simonstrator.api.component.resourcedistribution;

import de.tudarmstadt.maki.simonstrator.api.common.UniqueID;
import de.tudarmstadt.maki.simonstrator.api.component.HostComponent;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.location.Location;

/**
 * A node that provides resources to {@link ResourceConsumer}s. Access to this interface is usually only performed
 * in simulations.
 *
 * @author Tim Feuerbach
 */
public interface ResourceDistributor extends HostComponent {
    /**
     * Returns a globally unique identifier for this distributor node.
     */
    public UniqueID getBeaconID();

    /**
     * Returns the distributor's current sequence ID.
     */
    public int getBeaconSequenceID();

    /**
     * Returns the distributor's location.
     */
    public Location getLocation();

    /**
     * Returns a resource type identifier. If only one type of resources is used, a value of 0 is recommended, since
     * it may be expected by some simulation implementation defaults.
     */
    public short getResourceType();

    /**
     * Returns the number of resource units currently available at this distributor.
     */
    public int getResourceUnits();

    /**
     * Returns whether the resource distributor has no more resources available.
     */
    public boolean isEmpty();

    /**
     * Reduces the number of resources at this distributor by the given value. If there are less resource units
     * available, the distributor will only be reduced by the amount of remaining units.
     *
     * @param demand how many resources to "take"
     * @return how many resources the consumer actually got (0 <= got <= demand)
     */
    public int takeResources(int demand);

    /**
     * Adds a listener to this resource distributor.
     */
    public void registerListener(ResourceDistributorListener listener);
}
