/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of Simonstrator.KOM.
 * 
 * Simonstrator.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tudarmstadt.maki.simonstrator.api.component.resourcedistribution.analyzer;

import de.tudarmstadt.maki.simonstrator.api.component.core.MonitorComponent;
import de.tudarmstadt.maki.simonstrator.api.component.resourcedistribution.ResourceConsumer;

/**
 * Analyzer to capture when a {@link ResourceConsumer} expressed demand.
 * 
 * @author Tim Feuerbach
 */
public interface DemandAnalyzer extends MonitorComponent.Analyzer {

	/**
	 * Must be called by the selection strategy if demand has been noticed. This
	 * is typically done during in the checkDemand method. There are no
	 * guarantees on how early or how often this method will be called.
	 * 
	 * @param consumer
	 *            the consumer that noticed the demand
	 * @param resourceType
	 *            the resource type id, defaults to 0 for single resource
	 *            scenarios
	 * @param demand
	 *            the amount of resources required by the consumer; if not
	 *            specified otherwise by the strategy, this equals to the amount
	 *            of resources required to refill back to 100%
	 */
	public void demandNoticed(ResourceConsumer consumer, short resourceType, double demand);
}
